//oled096.c

#include <stdint.h>
#include <string.h>
#include "i2c.h"
#include "oled096.h" 

uint8_t oledBuf[1024];

void oledCommand(uint8_t comm) 
{
	i2c_WRIETE(0x00, &comm, 1);
}

void oledData(uint8_t data) 
{
	i2c_WRIETE(0x40, &data, 1);
}


void oledUpdate(void)
{
	uint16_t b = 0;
	oledCommand(SSD1306_SET_COLUMN_ADDR);
	oledCommand(0);
	oledCommand(127);
	oledCommand(SSD1306_SET_PAGE_ADDR);
	oledCommand(0);
	oledCommand(7);
	while(b < 1024)
	{
		oledData(oledBuf[b]);
		b++;
	}
}

void oledClear(void) //Эта функция очищает дисплей.
{
	memset(&oledBuf[0], 0, 1024);
}


void oledSetup(void) 
{
	 //Wire.begin();
	//процесс инициализации честно скопирован из даташита на дисплей.
	oledCommand(0xAE);//выключение дисплея
	oledCommand(0xD5);
	oledCommand(0x80);
	oledCommand(0xD3);
	oledCommand(0x0);
	oledCommand(0x40);
	oledCommand(0x8D);//включение емкостного умножителя
	oledCommand(0x14);
	oledCommand(0x20);//настройка адресации
	oledCommand(0x00);// 0х00 для горизонтальной, 0х01 для вертикальной, 0х02 для постраничной адресации
	oledCommand(0xA1);//отражение по горизонтали, для отображения справа налево необходимо использовать команду 0xA0
	oledCommand(0xC8);//отражение по вертикали, 0xC0 для переворота изображения по вертикали.
	//Одновременное использование команд 0xC8 и 0xA1 или 0xA0 и 0xC0 позволяет повернуть изображение на 180 градусов.
	oledCommand(0xDA);
	oledCommand(0x12);
	oledCommand(0x81);//установка контрастности дисплея
	oledCommand(0xCF);
	oledCommand(0xD9);
	oledCommand(0xF1);
	oledCommand(0xDB);
	oledCommand(0x40);
	oledCommand(0xA4);
	oledCommand(0xA6);//инверсия дисплея, 0xA6 для отключения инверсии, 0xA7 для включения инверсии цвета.
	oledCommand(0xAF);//включение дисплея
	oledClear();//Используем функцию очистки дисплея.
}

//
void oledInvert(uint8_t mode)
{
	if (mode)
		oledCommand(SSD1306_INVERT_DISPLAY);
	else
		oledCommand(SSD1306_NORMAL_DISPLAY);
}

//
void oledSetPixel(uint16_t x, uint16_t y)
{
	int by, bi;
	//
	if ((x >= 0) && (x < 128) && (y >= 0) && (y < 64))
	{
		by=((y / 8) * 128) + x;
		bi=y % 8;

		oledBuf[by]=oledBuf[by] | (1 << bi);
	}
}

void oledClrPixel(uint16_t x, uint16_t y)
{
	int by, bi;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		by=((y/8)*128)+x;
		bi=y % 8;

		oledBuf[by]=oledBuf[by] & ~(1<<bi);
	}
}

void oledInvPixel(uint16_t x, uint16_t y)
{
	int by, bi;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		by=((y/8)*128)+x;
		bi=y % 8;

		if ((oledBuf[by] & (1<<bi))==0)
			oledBuf[by]=oledBuf[by] | (1<<bi);
		else
			oledBuf[by]=oledBuf[by] & ~(1<<bi);
	}
}

void oledDrawHLine(int x, int y, int l)
{
	int by, bi, cx;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		for (cx=0; cx<l; cx++)
		{
			by=((y/8)*128)+x;
			bi=y % 8;

			oledBuf[by+cx] |= (1<<bi);
		}
	}
}

void oledClrHLine(int x, int y, int l)
{
	int by, bi, cx;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		for (cx=0; cx<l; cx++)
		{
			by=((y/8)*128)+x;
			bi=y % 8;

			oledBuf[by+cx] &= ~(1<<bi);
		}
	}
}

void oledDrawVLine(int x, int y, int l)
{
	int by, bi, cy;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		for (cy=0; cy<l; cy++)
		{
			oledSetPixel(x, y + cy);
		}
	}
}

void oledClrVLine(int x, int y, int l)
{
	int by, bi, cy;

	if ((x>=0) && (x<128) && (y>=0) && (y<64))
	{
		for (cy=0; cy<l; cy++)
		{
			oledClrPixel(x, y+cy);
		}
	}
}

void oledDrawLine(int x1, int y1, int x2, int y2)
{
	int tmp;
	int i;
	double delta, tx, ty;
	double m, b, dx, dy;
	
	if (((x2-x1)<0))
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
		tmp=y1;
		y1=y2;
		y2=tmp;
	}
    if (((y2-y1)<0))
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
		tmp=y1;
		y1=y2;
		y2=tmp;
	}

	if (y1==y2)
	{
		if (x1>x2)
		{
			tmp=x1;
			x1=x2;
			x2=tmp;
		}
		oledDrawHLine(x1, y1, x2-x1);
	}
	else if (x1==x2)
	{
		if (y1>y2)
		{
			tmp=y1;
			y1=y2;
			y2=tmp;
		}
		oledDrawVLine(x1, y1, y2-y1);
	}
	else if (abs(x2-x1)>abs(y2-y1))
	{
		delta=((double)(y2-y1) / (double)(x2-x1));
		ty=(double)(y1);
		if (x1>x2)
		{
			for (i=x1; i>=x2; i--)
			{
				oledSetPixel(i, (int)(ty+0.5));
        		ty=ty-delta;
			}
		}
		else
		{
			for (i=x1; i<=x2; i++)
			{
				oledSetPixel(i, (int)(ty+0.5));
        		ty=ty+delta;
			}
		}
	}
	else
	{
		delta=((float)(x2-x1)/(float)(y2-y1));
		tx=(float)(x1);
        if (y1>y2)
        {
			for (i=y2+1; i>y1; i--)
			{
		 		oledSetPixel((int)(tx+0.5), i);
        		tx=tx+delta;
			}
        }
        else
        {
			for (i=y1; i<y2+1; i++)
			{
		 		oledSetPixel((int)(tx+0.5), i);
        		tx=tx+delta;
			}
        }
	}

}

void oledClrLine(int x1, int y1, int x2, int y2)
{
	int tmp;
	int i;
	double delta, tx, ty;
	double m, b, dx, dy;
	
	if (((x2-x1)<0))
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
		tmp=y1;
		y1=y2;
		y2=tmp;
	}
    if (((y2-y1)<0))
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
		tmp=y1;
		y1=y2;
		y2=tmp;
	}

	if (y1==y2)
	{
		if (x1>x2)
		{
			tmp=x1;
			x1=x2;
			x2=tmp;
		}
		oledClrHLine(x1, y1, x2-x1);
	}
	else if (x1==x2)
	{
		if (y1>y2)
		{
			tmp=y1;
			y1=y2;
			y2=tmp;
		}
		oledClrVLine(x1, y1, y2-y1);
	}
	else if (abs(x2-x1)>abs(y2-y1))
	{
		delta=((double)(y2-y1)/(double)(x2-x1));
		ty=(double)(y1);
		if (x1>x2)
		{
			for (i=x1; i>=x2; i--)
			{
				oledClrPixel(i, (int)(ty+0.5));
        		ty=ty-delta;
			}
		}
		else
		{
			for (i=x1; i<=x2; i++)
			{
				oledClrPixel(i, (int)(ty+0.5));
        		ty=ty+delta;
			}
		}
	}
	else
	{
		delta=((float)(x2-x1)/(float)(y2-y1));
		tx=(float)(x1);
        if (y1>y2)
        {
			for (i=y2+1; i>y1; i--)
			{
		 		oledClrPixel((int)(tx+0.5), i);
        		tx=tx+delta;
			}
        }
        else
        {
			for (i=y1; i<y2+1; i++)
			{
		 		oledClrPixel((int)(tx+0.5), i);
        		tx=tx+delta;
			}
        }
	}

}

void oledDrawRect(int x1, int y1, int x2, int y2)
{
	int tmp;

	if (x1>x2)
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
	}
	if (y1>y2)
	{
		tmp=y1;
		y1=y2;
		y2=tmp;
	}

	oledDrawHLine(x1, y1, x2-x1);
	oledDrawHLine(x1, y2, x2-x1);
	oledDrawVLine(x1, y1, y2-y1);
	oledDrawVLine(x2, y1, y2-y1+1);
}

void oledClrRect(int x1, int y1, int x2, int y2)
{
	int tmp;

	if (x1>x2)
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
	}
	if (y1>y2)
	{
		tmp=y1;
		y1=y2;
		y2=tmp;
	}

	oledClrHLine(x1, y1, x2-x1);
	oledClrHLine(x1, y2, x2-x1);
	oledClrVLine(x1, y1, y2-y1);
	oledClrVLine(x2, y1, y2-y1+1);
}

void oledDrawRoundRect(int x1, int y1, int x2, int y2)
{
	int tmp;

	if (x1>x2)
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
	}
	if (y1>y2)
	{
		tmp=y1;
		y1=y2;
		y2=tmp;
	}
	if ((x2-x1)>4 && (y2-y1)>4)
	{
		oledSetPixel(x1+1,y1+1);
		oledSetPixel(x2-1,y1+1);
		oledSetPixel(x1+1,y2-1);
		oledSetPixel(x2-1,y2-1);
		oledDrawHLine(x1+2, y1, x2-x1-3);
		oledDrawHLine(x1+2, y2, x2-x1-3);
		oledDrawVLine(x1, y1+2, y2-y1-3);
		oledDrawVLine(x2, y1+2, y2-y1-3);
	}
}

void oledClrRoundRect(int x1, int y1, int x2, int y2)
{
	int tmp;

	if (x1>x2)
	{
		tmp=x1;
		x1=x2;
		x2=tmp;
	}
	if (y1>y2)
	{
		tmp=y1;
		y1=y2;
		y2=tmp;
	}
	if ((x2-x1)>4 && (y2-y1)>4)
	{
		oledClrPixel(x1+1,y1+1);
		oledClrPixel(x2-1,y1+1);
		oledClrPixel(x1+1,y2-1);
		oledClrPixel(x2-1,y2-1);
		oledClrHLine(x1+2, y1, x2-x1-3);
		oledClrHLine(x1+2, y2, x2-x1-3);
		oledClrVLine(x1, y1+2, y2-y1-3);
		oledClrVLine(x2, y1+2, y2-y1-3);
	}
}

void oledDrawCircle(int x, int y, int radius)
{
	int f = 1 - radius;
	int ddF_x = 1;
	int ddF_y = -2 * radius;
	int x1 = 0;
	int y1 = radius;
	char ch, cl;
	
	oledSetPixel(x, y + radius);
	oledSetPixel(x, y - radius);
	oledSetPixel(x + radius, y);
	oledSetPixel(x - radius, y);
 
	while(x1 < y1)
	{
		if(f >= 0) 
		{
			y1--;
			ddF_y += 2;
			f += ddF_y;
		}
		x1++;
		ddF_x += 2;
		f += ddF_x;    
		oledSetPixel(x + x1, y + y1);
		oledSetPixel(x - x1, y + y1);
		oledSetPixel(x + x1, y - y1);
		oledSetPixel(x - x1, y - y1);
		oledSetPixel(x + y1, y + x1);
		oledSetPixel(x - y1, y + x1);
		oledSetPixel(x + y1, y - x1);
		oledSetPixel(x - y1, y - x1);
	}
}

void oledClrCircle(int x, int y, int radius)
{
	int f = 1 - radius;
	int ddF_x = 1;
	int ddF_y = -2 * radius;
	int x1 = 0;
	int y1 = radius;
	char ch, cl;
	
	oledClrPixel(x, y + radius);
	oledClrPixel(x, y - radius);
	oledClrPixel(x + radius, y);
	oledClrPixel(x - radius, y);
 
	while(x1 < y1)
	{
		if(f >= 0) 
		{
			y1--;
			ddF_y += 2;
			f += ddF_y;
		}
		x1++;
		ddF_x += 2;
		f += ddF_x;    
		oledClrPixel(x + x1, y + y1);
		oledClrPixel(x - x1, y + y1);
		oledClrPixel(x + x1, y - y1);
		oledClrPixel(x - x1, y - y1);
		oledClrPixel(x + y1, y + x1);
		oledClrPixel(x - y1, y + x1);
		oledClrPixel(x + y1, y - x1);
		oledClrPixel(x - y1, y - x1);
	}
}



